module GenerationStateMonad where

import Formula

newtype StateMonad s a = SM (s -> (a, s))

instance Functor (StateMonad s) where
  map f (SM m) = SM (\s -> let (x, s') = m s
                           in (f x, s'))

instance Monad (StateMonad s) where
  return x   = SM (\s -> (x, s))
  SM m >>= f = SM (\s -> let (x, s') = m s
                             SM f'  = f x
                         in f' s')


startingWith :: StateMonad s a -> s -> a
(SM m) `startingWith` v = x
  where (x,_) = m v

fetch :: StateMonad s s
fetch = SM (\s -> (s,s))

fetchWith :: (s -> a) -> StateMonad s a
fetchWith f = SM (\s -> (f s, s))

update :: (s -> s) -> StateMonad s s
update f = SM (\s -> (s, f s))

set :: s -> StateMonad s s
set s' = SM (\s -> (s, s'))

---------------


type GenSM a = StateMonad Generation a

instance (Show s, Num s, Show a) => Show (StateMonad s a) where
  showsPrec _ (SM m) = showString "State: " . shows s . showString ", Value: " .
                       shows v
    where (v,s) = m 0


nextGeneration :: GenSM Generation
nextGeneration = update (+1)

getGeneration :: GenSM Generation
getGeneration  = fetch
